/*
    Copyright 2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef WIN32
    #define stricmp strcasecmp
    #define stristr strcasestr
#endif

#ifndef u_char
    typedef unsigned char   u_char;
#endif


#define VER     "0.1"



void delimit(u_char *data);
void komamail_accounts_dec(u_char *pwd, int len);
void komamail_pwd(u_char *pwd, u_char *key);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *fd;
    int     i,
            len;
    u_char  buff[8192],
//            pass[512],
            *fname,
            *p;


    setbuf(stdout, NULL);

    fputs("\n"
        "KomaMail files and passwords decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <file>\n"
            "\n"
            "File can be Setup.ini or Accounts.dat available in the user's folder.\n"
            "\n", argv[0]);
        fputs("\n  press RETURN to exit\n", stdout);
        fgetc(stdin);
        exit(1);
    }

    fname = argv[1];
    fd = fopen(fname, "rb");
    if(!fd) std_err();

    p = strrchr(fname,
#ifdef WIN32
        '\\');
#else
        '/');
#endif
    if(p) fname = p + 1;

    if(!stricmp(fname, "setup.ini")) {
        for(i = 0; i < 26; i++) {
            fgets(buff, sizeof(buff), fd);
            delimit(buff);
        }
        komamail_pwd(buff, NULL);
        printf("Password:   %s\n", buff);

    } else if(!stricmp(fname, "accounts.dat")) {
        for(;;) {
            for(i = 0; i < 7; i++) {
                len = fgetc(fd);
                if((len < 0) || !fread(buff, 100, 1, fd)) break;
                komamail_accounts_dec(buff, len);
                switch(i) {
                    case 0: printf("  Account Name:      "); break;
                    case 1: printf("  Name:              "); break;
                    case 2: printf("  Mail-Address:      "); break;
                    case 3: printf("  Login Name:        "); break;
                    case 4: printf("  Password:          "); break;
                    case 5: printf("  Login for smtp:    "); break;
                    case 6: printf("  Password for smtp: "); break;
                    default: break;
                }
                printf("  %s\n", buff);
            }
            if(feof(fd)) break;
            if(fseek(fd, 2497, SEEK_CUR) < 0) break;
            fputc('\n', stdout);
        }

/* works with mailhead.dat but it's useless
    } else if(stristr(fname, ".dat")) {
        fputs(
            "- insert the required password if this file is encrypted:\n"
            "  (the password is stored in the file setup.ini, read it with this tool)\n"
            "  ", stdout);
        fflush(stdin);
        fgets(pass, sizeof(pass), stdin);
        delimit(pass);

        while(fgets(buff, sizeof(buff), fd)) {
            delimit(buff);
            komamail_pwd(buff, pass);
            printf("%s\n", buff);
        }
*/
    } else {
        fputs("  File not supported by this program\n", stdout);
    }

    fclose(fd);
    fputs("\n  press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



void delimit(u_char *data) {
    for(; *data && (*data != '\r') && (*data != '\n'); data++);
    *data = 0;
}



void komamail_accounts_dec(u_char *pwd, int len) {
    const static u_char key[] = "sdfgsdtgerthtzhfg",
            *p;

    p = key;
    while(len--) {
        *pwd -= *p;
        pwd++;
        p++;
        if(!*p) p = key;
    }
    *pwd = 0;
}



void komamail_pwd(u_char *pwd, u_char *key) {
    int     chr;
    const static u_char keyfix[] = "fw4523ertsdg345",
            *p;

    if(!key) key = (u_char *)keyfix;
    p = key;
    while(*pwd) {
        if((char)(*pwd + 128) < 0) {
            chr = (*pwd - *p) % 95;
            while(chr < 0) chr += 95;
            *pwd = chr + 32;
        }

        pwd++;
        p++;
        if(!*p) p = key;
    }
}



void std_err(void) {
    perror("\nError");
    fputs("\n  press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}


